package com.zehong.dangerousdevicereport.netty.handler;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.zehong.dangerousdevicereport.entity.SafetyDeviceInfo;
import com.zehong.dangerousdevicereport.entity.TAlarmInfo;
import com.zehong.dangerousdevicereport.entity.TDeviceReportData;
import com.zehong.dangerousdevicereport.enums.DeviceStatusEnum;
import com.zehong.dangerousdevicereport.service.ISafetyDeviceInfoService;
import com.zehong.dangerousdevicereport.service.ITAlarmInfoService;
import com.zehong.dangerousdevicereport.service.ITDeviceReportDataService;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import io.netty.util.CharsetUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
@Sharable
public class UDPServerHandler extends SimpleChannelInboundHandler<DatagramPacket> {

    private Logger logger = LoggerFactory.getLogger(UDPServerHandler.class);

    @Resource
    private ITDeviceReportDataService tDeviceReportDataService;
    @Resource
    private ITAlarmInfoService itAlarmInfoService;
    @Resource
    private ISafetyDeviceInfoService safetyDeviceInfoService;

    /**
     * msg格式为：设备编号/设备状态/设备浓度/
     *多组数据，以@分割
     * @param ctx
     * @param msg 上报数据
     * @throws Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket msg) throws Exception {

        //接收上传数据
        ByteBuf byteBuf = msg.content();
        String bufStr = byteBuf.toString(CharsetUtil.UTF_8);
        //校验设备上传数据
        if(StrUtil.isEmpty(bufStr) && !bufStr.contains("/")){
            logger.error("接收数据异常！");
            throw new Exception("接收数据异常，请联系管理员查看设备上传数据！");
        }
        logger.info("udp接收上传数据：" + bufStr);
        bufStr=bufStr.trim();
        //上传数据入库并处理报警信息
        List<TDeviceReportData> dataList = saveReportDataAndDealAlarmInfo(bufStr);
        //报警信息处理
        for(TDeviceReportData data : dataList){
            dealAlarmInfo(data);
        }
        //数据上报市局

        //数据上报省厅
    }

    /**
     * 上报数据入库
     * @param bufStr 上报数据
     */
    private List<TDeviceReportData> saveReportDataAndDealAlarmInfo(String bufStr) {

        List<TDeviceReportData> datas = new ArrayList<>();
        //单组数据上报
        if(!bufStr.contains("@")){
            TDeviceReportData param = assembleSaveInfo(bufStr);
            tDeviceReportDataService.insertTDeviceReportData(param);
            datas.add(param);
            return datas;
        }
        //多组数据上报
        String[] reportDataInfos = bufStr.split("@");
        for(String data : reportDataInfos){
            TDeviceReportData param = assembleSaveInfo(data);
            tDeviceReportDataService.insertTDeviceReportData(param);
            datas.add(param);
        }
        return datas;
    }

    /**
     * 处理报警信息
     * @param data 报警信息
     */
    private void dealAlarmInfo(TDeviceReportData data){
        //查询报警信息是否存在
        TAlarmInfo tAlarmInfo = itAlarmInfoService.selectAlarmInfoByDeviceNum(data.getDeviceNum());
        if(null != tAlarmInfo){
            if(DeviceStatusEnum.PREHEAT.getCode().equals(data.getDeviceStatus()) || DeviceStatusEnum.NORMAL.getCode().equals(data.getDeviceStatus())){
                tAlarmInfo.setHandleTime(DateUtil.date());
                tAlarmInfo.setAlarmStatus("1");
                tAlarmInfo.setHandleType("1");
                itAlarmInfoService.updateTAlarmInfo(tAlarmInfo);
                return;
            }
            //设备报警信息未正常更新报警信息
            if(!tAlarmInfo.getAlarmInfo().equals(data.getDeviceStatus())){
                tAlarmInfo.setAlarmInfo(data.getDeviceStatusDescribe());
                tAlarmInfo.setReportValue(data.getDevicePotency());
                tAlarmInfo.setAlarmTime(DateUtil.date());
                itAlarmInfoService.updateTAlarmInfo(tAlarmInfo);
                return;
            }
            return;
        }

        //新增报警信息
        if(!DeviceStatusEnum.PREHEAT.getCode().equals(data.getDeviceStatus()) && !DeviceStatusEnum.NORMAL.getCode().equals(data.getDeviceStatus())){
            insertAlarmInfo(data);
        }

    }

    /**
     * 新增报警信息
     * @param data 上报数据
     */
    private void insertAlarmInfo(TDeviceReportData data){
        try{
            SafetyDeviceInfo safetyDeviceInfo = getSafeDeviceInfo(data.getDeviceNum());
            TAlarmInfo alarmInfo = new TAlarmInfo();
            alarmInfo.setId(IdUtil.simpleUUID());
            alarmInfo.setEnterpriseId(safetyDeviceInfo.getEnterpriseId());
            alarmInfo.setDeviceId(safetyDeviceInfo.getId());
            alarmInfo.setAlarmInfo(data.getDeviceStatusDescribe());
            alarmInfo.setReportValue(data.getDevicePotency());
            alarmInfo.setAlarmTime(DateUtil.date());
            itAlarmInfoService.insertTAlarmInfo(alarmInfo);
        }catch (Exception e){
            logger.error("报警信息新增失败:"+e);
        }

    }

    /**
     * 获取设备信息
     * @param deviceNum 设备编号
     * @return SafetyDeviceInfo 设备信息
     * @throws Exception
     */
    private SafetyDeviceInfo getSafeDeviceInfo(String deviceNum) throws Exception {
        SafetyDeviceInfo safetyDeviceInfo = new SafetyDeviceInfo();
        safetyDeviceInfo.setDeviceNumber(deviceNum);
        List<SafetyDeviceInfo> safetyDeviceInfos = safetyDeviceInfoService.selectSafetyDeviceInfoList(safetyDeviceInfo);
        if(null == safetyDeviceInfos || safetyDeviceInfos.size()==0 || safetyDeviceInfos.size()>1){
            throw new Exception("通知管理员设备不存在设备编号重复，请核对！");
        }
        return safetyDeviceInfos.get(0);
    }

    /**
     * 拼装上报数据入库参数
     * @param reportData 上报数据
     * @return TDeviceReportData
     */
    private TDeviceReportData assembleSaveInfo(String reportData){
        String[] data = reportData.split("/");
        TDeviceReportData tDeviceReportData = new TDeviceReportData();
        tDeviceReportData.setDeviceNum(data[0].trim());
        tDeviceReportData.setDeviceStatus(data[1].trim());
        tDeviceReportData.setDeviceStatusDescribe(deviceStatus(data[1].trim()));
        tDeviceReportData.setDevicePotency(data[2].trim().indexOf(".")==0?"0"+data[2].trim():data[2].trim());

        return tDeviceReportData;
    }

    /**
     * 获取设备状态
     * @param code 设备状态编码
     * @return String
     */
    private String deviceStatus(String code){
       DeviceStatusEnum[] value = DeviceStatusEnum.values();
       for(DeviceStatusEnum deviceStatusEnum : value){
           if(deviceStatusEnum.getCode().equals(code)){
               return deviceStatusEnum.getStatus();
           }
       }
       return "";
    }
}
